#include <konkret/konkret.h>
#include "Linux_MemberOfGroup.h"
#include "Linux_Group.h"
#include "Linux_Identity.h"

#include "aux_lu.h"
#include "globals.h"

#include <libuser/entity.h>
#include <libuser/user.h>

static const CMPIBroker* _cb;

static void Linux_MemberOfGroupInitialize()
{
}

static CMPIStatus Linux_MemberOfGroupCleanup( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_MemberOfGroupEnumInstanceNames( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Linux_MemberOfGroupEnumInstances( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    Linux_GroupRef lgref;
    Linux_IdentityRef liref;
    Linux_MemberOfGroup lmog;

    struct lu_context *luc = NULL;
    struct lu_error *error = NULL;
    GPtrArray *groups = NULL;
    GValueArray *accounts = NULL;
    struct lu_ent *lueg = NULL;
    struct lu_ent *luea = NULL;

    size_t i, j;
    const char *nameSpace = KNameSpace(cop);
    char *uid = NULL;

    luc = lu_start(NULL, 0, NULL, NULL, lu_prompt_console_quiet, NULL, &error);
    if (!luc)
      {
        fprintf(stderr, "Error initializing: %s\n", lu_strerror(error));
        exit(1);
      }
    uid = malloc(256 * sizeof(char));
    /* Go through each group */
    groups = lu_groups_enumerate_full(luc, "*", &error);
    for (i = 0;  (groups != NULL) && (i < groups->len); i++)
      {
        lueg = g_ptr_array_index(groups, i);
        Linux_GroupRef_Init(&lgref, _cb, nameSpace);
        Linux_GroupRef_Set_CreationClassName(&lgref, LGCCLASSNAME);
        Linux_GroupRef_Set_Name(&lgref, aux_lu_get_str(lueg, LU_GROUPNAME));

        /* For each user in the group */
        accounts = lu_users_enumerate_by_group(luc,
          aux_lu_get_str(lueg, LU_GROUPNAME), &error);
        for (j = 0;  (accounts != NULL) && (j < account->n_values); j++)
          {
            luea = lu_users_enumerate_full(luc,
              g_array_index(accounts, char *, j), &error); /* XXX */
            lu_ent_free(luea);
          } /* for users */
        lu_ent_free(lueg);
      } /* for groups */
    free(uid);

    if (groups)
      {
        g_ptr_array_free(groups, TRUE);
      }

    lu_end(luc);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_MemberOfGroupGetInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc,
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Linux_MemberOfGroupCreateInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const CMPIInstance* ci) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_MemberOfGroupModifyInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop,
    const CMPIInstance* ci, 
    const char**properties) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_MemberOfGroupDeleteInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_MemberOfGroupExecQuery(
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char* lang, 
    const char* query) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Linux_MemberOfGroupAssociationCleanup( 
    CMPIAssociationMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term) 
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Linux_MemberOfGroupAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_MemberOfGroup_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Linux_MemberOfGroupAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_MemberOfGroup_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Linux_MemberOfGroupReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_MemberOfGroup_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Linux_MemberOfGroupReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Linux_MemberOfGroup_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub( 
    Linux_MemberOfGroup,
    Linux_MemberOfGroup,
    _cb,
    Linux_MemberOfGroupInitialize())

CMAssociationMIStub( 
    Linux_MemberOfGroup,
    Linux_MemberOfGroup,
    _cb,
    Linux_MemberOfGroupInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_MemberOfGroup",
    "Linux_MemberOfGroup",
    "instance association");
